//
//  OANetworkManager.h
//  OAHabraManager
//
//  Created by Orlov Alexander on 01.05.13.
//  Copyright (c) 2013 Orlov Alexander. All rights reserved.
//

#import "AFNetworking.h"
#import "GDataXMLNode.h"

#import "NetworkConfiguration.h"

#import "PostCutted.h"
#import "PostFull.h"

#import "QuestionCutted.h"
#import "QuestionFull.h"

@interface OAHabraManager : AFHTTPClient

#pragma mark - Singleton Methods
+ (instancetype) sharedManager;

#pragma mark - Functional REST Methods
///---------------------------------------
/// @name Authorization
///---------------------------------------

///---------------------------------------
/// @name Feeds
///---------------------------------------

#pragma mark - Fetching 'Posts'
///---------------------------------------
/// @name Posts
///---------------------------------------
/**
 Creates request to the site and fetches posts list.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else postsCutted object represents mapped entities.
 @param pSource Type of content source on the site. For the now site have many sources of content. Look definition or example.
 @param page This is 'int' variable thats points to page number on the site results in selected by 'pSource' content source. This parameter must be 1 or more than 1.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
 */
- (AFHTTPRequestOperation*) getPostsCuttedListWithBlock:(void (^)(NSArray *errors, NSArray *postsCutted))block
                                                 fromSource:(OAPostsSourceType)pSource
                                                    andPage:(int)page
                                           startImmediately:(BOOL)immediately;
/**
 Creates request to the site and fetches full version of your desired post.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else questionsCutted object represents mapped entities.
 @param revealPath Reveal to service URL path to desired post. For example @"/post/187334/". This parameter must not be nil.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
 */
- (AFHTTPRequestOperation*) getPostFullWithBlock:(void (^)(NSArray *errors, PostFull *fullPost))block
                                 fromRevealURLString:(NSString*)revealPath
                                    startImmediately:(BOOL)immediately;
/**
 Creates request to the site and fetches full version of your desired question.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else questionsCutted object represents mapped entities.
 @param postHeader It is object of type 'PostCutted', that contains many short infos, and fetch form this revealPath to desired post. So, this parameter must not be nil.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
 */
- (AFHTTPRequestOperation*) getPostFullWithBlock:(void (^)(NSArray *errors, PostFull *fullPost))block
                                  fromCuttedPost:(PostCutted*)postHeader
                                    startImmediately:(BOOL)immediately;
#pragma mark - Fetching 'Questions and Answers'
///---------------------------------------
/// @name Questions and Answers
///---------------------------------------
/**
 Creates request to the site and fetches cutted questions list.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else questionsCutted object represents mapped entities.
 @param qSource Type of content source on the site. For the now site have many sources of content. Look definition or example.
 @param page This is 'int' variable thats points to page number on the site results in selected by 'qSource' content source. This parameter must be 1 or more than 1.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
*/
- (AFHTTPRequestOperation*) getQuestionsCuttedListWithBlock:(void (^)(NSArray *errors, NSArray *questionsCutted))block
                                                 fromSource:(OAQuestionsSourceType)qSource
                                                    andPage:(int)page
                                           startImmediately:(BOOL)immediately;
/**
 Creates request to the site and fetches full version of your desired question.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else questionsCutted object represents mapped entities.
 @param revealPath Reveal to service URL path to desired question. For example @"/qa/43714/". This parameter must not be nil.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
 */
- (AFHTTPRequestOperation*) getQuestionFullWithBlock:(void (^)(NSArray *errors, QuestionFull *fullQuestion))block
                                 fromRevealURLString:(NSString*)revealPath
                                    startImmediately:(BOOL)immediately;
/**
 Creates request to the site and fetches full version of your desired question.
 
 @param block Your working block to work with site response. If error object not nil thats mean that opeartion has been failed. Else questionsCutted object represents mapped entities.
 @param questionHeader It is object of type 'QuestionCutted', that contains many short infos, and fetch form this revealPath to desired question. So, this parameter must not be nil.
 @param immediately This parameter determines will operation starts immediately after creating, or not. You can set up this to NO to manage operations by yourself.
 
 @return AFHTTPRequestOperation thats present fetching request to the site.
 */
- (AFHTTPRequestOperation*) getQuestionFullWithBlock:(void (^)(NSArray *errors, QuestionFull *fullQuestion))block
                                 fromCuttedQuestion:(QuestionCutted*)questionHeader
                                    startImmediately:(BOOL)immediately;

@end
