//
//  OANetworkManager.m
//  OAHabraManager
//
//  Created by Orlov Alexander on 01.05.13.
//  Copyright (c) 2013 Orlov Alexander. All rights reserved.
//

#import "OAHabraManager.h"

@implementation OAHabraManager

#pragma mark - Singleton Methods
+ (instancetype) sharedManager
{
    static OAHabraManager *_sharedClient = nil;
    static dispatch_once_t onceToken;
    
    dispatch_once(&onceToken, ^{
        _sharedClient = [[OAHabraManager alloc] initWithBaseURL:[NSURL URLWithString:kServiceURL]];
    });
    
    return _sharedClient;
}

#pragma mark - Internal Methods
- (instancetype) initWithBaseURL:(NSURL *)url {
    
    self = [super initWithBaseURL:url];
    
    if (!self) {
        return nil;
    }
    
    [self registerHTTPOperationClass:[AFXMLRequestOperation class]];
	[self setDefaultHeader:@"Accept" value:@"application/html"];
    [self setParameterEncoding:AFFormURLParameterEncoding];
    
    return self;
}

#pragma mark - Utils Methods
- (NSArray*) commentsToPostFromNode:(GDataXMLElement*)nodeThatContainsComments
{
    NSMutableArray *comments = [@[] mutableCopy];
    
    NSError  *commentsError = nil;
    NSString *commentsQueryString       = @"div[contains(@class,'comment_item')]";
    NSArray  *commentsNodes     = [nodeThatContainsComments nodesForXPath:commentsQueryString error:&commentsError];
    
    if (commentsError)
    {
        return nil;
    }
    
    for (GDataXMLElement *commentNode in commentsNodes)
    {
        NSMutableDictionary *commentDictionary = [@{} mutableCopy];
        NSMutableDictionary *authorDictionary = [@{} mutableCopy];
        NSArray *replyComments = [@[] mutableCopy];
        
        NSError  *authorAvatarError = nil;
        NSString *authorAvatarQueryString       = @"div[@class='comment_body']/div[contains(@class,'info')]/*[contains(@class,'avatar')]/img";
        GDataXMLElement  *authorAvatarNode      = (GDataXMLElement*)[commentNode firstNodeForXPath:authorAvatarQueryString error:&authorAvatarError];
        
        NSString *authorAvatarString = [[authorAvatarNode attributeForName:@"src"] stringValue];
        if (authorAvatarString)
        {
            [authorDictionary setObject:authorAvatarString forKey:@"avatar"];
        }
        
        NSError  *authorNickError = nil;
        NSString *authorNickQueryString       = @"div[@class='comment_body']/div[contains(@class,'info')]/*[contains(@class,'username')]";
        GDataXMLElement  *authorNickNode      = (GDataXMLElement*)[commentNode firstNodeForXPath:authorNickQueryString error:&authorNickError];
        
        NSString *authorNickString = [authorNickNode stringValue];
        if (authorNickString)
        {
            [authorDictionary setObject:authorNickString forKey:@"username"];
        }
        
        NSError  *timeError = nil;
        NSString *timeQueryString       = @"div[@class='comment_body']/div[contains(@class,'info')]/time";
        GDataXMLElement  *timeNode      = (GDataXMLElement*)[commentNode firstNodeForXPath:timeQueryString error:&timeError];
        
        NSString *timeString = [timeNode stringValue];
        if (timeString)
        {
            [commentDictionary setObject:timeString forKey:@"time"];
        }
        
        NSError  *messageBodyError = nil;
        NSString *messageBodyQueryString       = @"div[@class='comment_body']/*[contains(@class,'message html_format')]";
        GDataXMLElement  *messageBodyNode      = (GDataXMLElement*)[commentNode firstNodeForXPath:messageBodyQueryString error:&messageBodyError];
        
        NSString *messageBodyString = [messageBodyNode stringValue];
        if (messageBodyString)
        {
            [commentDictionary setObject:messageBodyString forKey:@"body"];
        }
        
        NSError  *replyCommentsError = nil;
        NSString *replyCommentsQueryString       = @"div[@class='reply_comments']";
        GDataXMLElement  *replyCommentsNode      = (GDataXMLElement*)[commentNode firstNodeForXPath:replyCommentsQueryString error:&replyCommentsError];
        
        if (replyCommentsNode)
        {
            replyComments = [self commentsToPostFromNode:replyCommentsNode];//to understand recursion, you must understand recursion  ;)
        }
        
        [commentDictionary setObject:replyComments forKey:@"comments"];
        [commentDictionary setObject:authorDictionary forKey:@"author"];
        [comments addObject:commentDictionary];
    }
    
    return comments;
}

#pragma mark - Fetching 'Posts'
- (AFHTTPRequestOperation*) getPostsCuttedListWithBlock:(void (^)(NSArray *errors, NSArray *postsCutted))block
                                             fromSource:(OAPostsSourceType)pSource
                                                andPage:(int)page
                                       startImmediately:(BOOL)immediately
{
    NSMutableArray *errorsOccured = [@[] mutableCopy];
    
    if (page < 1)
    {
        if (block)
        {
            NSError *errorWrongPage = [NSError errorWithDomain:kDomain code:-1 userInfo:@{NSLocalizedDescriptionKey:NSLocalizedString(@"Passed 'page' value is less than 1", "")}];
            [errorsOccured addObject:errorWrongPage];
            
            block(errorsOccured, nil);
        }
        
        return nil;
    }
    
    NSString *urlString;
    NSArray  *postsSources = @[kPostsSourceBestDaily, kPostsSourceBestWeekly, kPostsSourceBestMonthly, kPostsSourceBestAll,
                               kPostsSourceThematicHabred, kPostsSourceThematicNew, kPostsSourceThematicUnHabred, kPostsSourceThematicSandbox,
                               kPostsSourceCorporativeHabred, kPostsSourceCorporativeNew, kPostsSourceCorporativeUnHabred];
    
    urlString = [postsSources objectAtIndex:pSource];
    urlString = [urlString stringByAppendingFormat:kPostsSourcePageFormatPostfix, page];
    
    NSURLRequest *postsCuttedGetRequest = [self requestWithMethod:@"GET" path:urlString parameters:nil];
    AFHTTPRequestOperation *postsCuttedGetOperation = [self HTTPRequestOperationWithRequest:postsCuttedGetRequest success:^(AFHTTPRequestOperation *operation, id responseObject) {
        
        if (block)
        {
            NSError *errorWhenParse = nil;
            
            NSData   *postsSourceHTMLData = responseObject;
            GDataXMLDocument *cuttedPostsDocument  = [[GDataXMLDocument alloc] initWithHTMLData:postsSourceHTMLData error:&errorWhenParse];
            
            if (errorWhenParse)
            {
                [errorsOccured addObject:errorWhenParse];
                block(errorsOccured, nil);
                
                return;
            }
            
            NSError  *postsError = nil;
            NSString *postsQueryString       = @"//div[@class='posts_list']/div[@class='posts shortcuts_items']/div[contains(@class,'post')]";
            NSArray  *postsNodes             = [cuttedPostsDocument nodesForXPath:postsQueryString error:&postsError];
            
            if (postsError)
            {
                [errorsOccured addObject:postsError];
                block(errorsOccured, nil);
                
                return;
            }
            
            NSMutableArray *newPostsCutted = [NSMutableArray array];
            
            for (GDataXMLElement *postNode in postsNodes)
            {
                PostCutted *newPostHeader = [[PostCutted alloc] init];
                
                NSError  *titleError = nil;
                NSString *titleQueryString       = @"h1[@class='title']/a[@class='post_title']";
                GDataXMLElement  *title          = (GDataXMLElement*)[postNode firstNodeForXPath:titleQueryString error:&titleError];
                
                if (titleError)
                {
                    [errorsOccured addObject:titleError];
                }
                
                NSError  *isTranslateError = nil;
                NSString *isTranslateQueryString       = @"h1[@class='title']/*[contains(@class, 'flag flag_translation')]";
                GDataXMLElement  *isTranslation        = (GDataXMLElement*)[postNode firstNodeForXPath:isTranslateQueryString error:&isTranslateError];
                
                if (isTranslateError)
                {
                    [errorsOccured addObject:isTranslateError];
                }
                
                NSError  *isRecoveryModeError = nil;
                NSString *isRecoveryModeQueryString    = @"h1[@class='title']/*[contains(@class, 'flag flag_recovery')]";
                GDataXMLElement  *isRecoveryMode       = (GDataXMLElement*)[postNode firstNodeForXPath:isRecoveryModeQueryString error:&isRecoveryModeError];
                
                if (isRecoveryModeError)
                {
                    [errorsOccured addObject:isRecoveryModeError];
                }
                
                NSError  *isTutorialError = nil;
                NSString *isTutorialQueryString       = @"h1[@class='title']/*[contains(@class, 'flag flag_tutorial')]";
                GDataXMLElement  *isTutorial          = (GDataXMLElement*)[postNode firstNodeForXPath:isTutorialQueryString error:&isTutorialError];
                
                if (isTutorialError)
                {
                    [errorsOccured addObject:isTutorialError];
                }
                
                NSError  *previewError = nil;
                NSString *previewQueryString         = @"div[contains(@class,'content html_format')]";
                GDataXMLElement  *preview            = (GDataXMLElement*)[postNode firstNodeForXPath:previewQueryString error:&previewError];
                
                if (previewError)
                {
                    [errorsOccured addObject:previewError];
                }
                
                NSError  *timeError = nil;
                NSString *timeQueryString         = @"div[@class='published']";
                GDataXMLElement  *time            = (GDataXMLElement*)[postNode firstNodeForXPath:timeQueryString error:&timeError];
                
                if (timeError)
                {
                    [errorsOccured addObject:timeError];
                }
                
                NSError  *hubsError = nil;
                NSString *hubsQueryString         = @"div[@class='hubs']";
                GDataXMLElement  *hubs            = (GDataXMLElement*)[postNode firstNodeForXPath:hubsQueryString error:&hubsError];
                
                if (hubsError)
                {
                    [errorsOccured addObject:hubsError];
                }
                
                NSError  *viewsError = nil;
                NSString *viewsQueryString        = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='pageviews']";
                GDataXMLElement  *views           = (GDataXMLElement*)[postNode firstNodeForXPath:viewsQueryString error:&viewsError];
                
                if (viewsError)
                {
                    [errorsOccured addObject:viewsError];
                }
                
                NSError  *favAddsError = nil;
                NSString *favAddsQueryString      = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='favs_count']";
                GDataXMLElement  *favAdds         = (GDataXMLElement*)[postNode firstNodeForXPath:favAddsQueryString error:&favAddsError];
                
                if (favAddsError)
                {
                    [errorsOccured addObject:favAddsError];
                }
                
                NSError  *authorError = nil;
                NSString *authorQueryString       = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='author']/a";
                GDataXMLElement  *author          = (GDataXMLElement*)[postNode firstNodeForXPath:authorQueryString error:&authorError];
                
                if (authorError)
                {
                    [errorsOccured addObject:authorError];
                }
                
                NSError  *commentsError = nil;
                NSString *commentsQueryString      = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[contains(@class,'comments')]/a/span[@class='all']";
                GDataXMLElement  *comments         = (GDataXMLElement*)[postNode firstNodeForXPath:commentsQueryString error:&commentsError];
                
                if (commentsError)
                {
                    [errorsOccured addObject:commentsError];
                }
                
                NSError  *ratingError = nil;
                NSString *ratingQueryString      = @"a[@class='score']|//span[@class='score']";
                GDataXMLElement  *rating         = (GDataXMLElement*)[postNode firstNodeForXPath:ratingQueryString error:&ratingError];
                
                if (ratingError)
                {
                    [errorsOccured addObject:ratingError];
                }
                
                newPostHeader.favAdds = [favAdds stringValue];
                
                if (newPostHeader.favAdds.length == 0)
                {
                    newPostHeader.favAdds = @"0";
                }
                
                NSString *hubsString = @"";
                NSArray *hubsElements = [hubs children];
                
                for (GDataXMLElement *hubElement in hubsElements)
                {
                    NSString *stringToAdd = [[hubElement childAtIndex:0] stringValue];
                    
                    if ((stringToAdd.length>0) & !([stringToAdd isEqual:@"*"]))
                    {
                        hubsString = [hubsString stringByAppendingFormat:@"%@; ",stringToAdd];
                    }
                }
                
                newPostHeader.tags    = hubsString;
                newPostHeader.title   = [title stringValue];
                
                NSString *fullAdress = [[title attributeForName:@"href"] stringValue];
                NSString *revealAdress = [fullAdress substringFromIndex:[fullAdress rangeOfString:@"http://habrahabr.ru"].location+[fullAdress rangeOfString:@"http://habrahabr.ru"].length];
                newPostHeader.url     =  revealAdress;
                
                newPostHeader.preview = [preview XMLString];
                newPostHeader.date    = [time stringValue];
                newPostHeader.rating  = [rating stringValue];
                newPostHeader.views   = [views stringValue];
                newPostHeader.author  = [author stringValue];
                newPostHeader.comments = [[comments stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
                
                if ([newPostHeader.comments isEqualToString:@"Комментировать"])
                {
                    newPostHeader.comments = @"0";
                }
                
                if (isRecoveryMode)
                {
                    newPostHeader.isRecoveryMode = YES;
                }
                
                if (isTutorial)
                {
                    newPostHeader.isTutorial = YES;
                }
                
                if (isTranslation)
                {
                    newPostHeader.isTranslate = YES;
                }
                
                [newPostsCutted addObject:newPostHeader];
            }
            
            block (errorsOccured, newPostsCutted);
        }
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        
        if (block)
        {
            block(@[error], nil);
        }
        
    }];
    
    if (immediately)
    {
        [self enqueueHTTPRequestOperation:postsCuttedGetOperation];
    }
    
    return postsCuttedGetOperation;
}

- (AFHTTPRequestOperation*) getPostFullWithBlock:(void (^)(NSArray *errors, PostFull *fullPost))block
                             fromRevealURLString:(NSString*)revealPath
                                startImmediately:(BOOL)immediately
{
    NSMutableArray *errorsOccured = [@[] mutableCopy];
    
    if (revealPath.length == 0)
    {
        if (block)
        {
            NSError *errorPathIsNil = [NSError errorWithDomain:kDomain code:-1 userInfo:@{NSLocalizedDescriptionKey:NSLocalizedString(@"Reveal path to post must not be nil", "")}];
            [errorsOccured addObject:errorPathIsNil];
            
            block(errorsOccured, nil);
        }
        
        return nil;
    }
    
    NSURLRequest *postFullGetRequest = [self requestWithMethod:@"GET" path:revealPath parameters:nil];
    AFHTTPRequestOperation *postFullGetOperation = [self HTTPRequestOperationWithRequest:postFullGetRequest success:^(AFHTTPRequestOperation *operation, id responseObject) {
        
        if (block)
        {
            NSError *errorWhenParse = nil;
            
            NSData   *postFullSourceHTMLData = responseObject;
            GDataXMLDocument *fullPostDocument  = [[GDataXMLDocument alloc] initWithHTMLData:postFullSourceHTMLData error:&errorWhenParse];
            
            if (errorWhenParse)
            {
                [errorsOccured addObject:errorWhenParse];
                block(errorsOccured, nil);
                
                return;
            }
            
            NSError  *postError = nil;
            NSString *postQueryString       = @"//div[contains(@id,'post')]";
            GDataXMLElement  *postNode     = (GDataXMLElement*)[fullPostDocument firstNodeForXPath:postQueryString error:&postError];
            
            if (postError)
            {
                [errorsOccured addObject:postError];
                block(errorsOccured, nil);
                
                return;
            }
            
            NSError  *commentsError = nil;
            NSString *commentsQueryString       = @"//div[contains(@class,'comments_list')]";
            GDataXMLElement  *commentsNode     = (GDataXMLElement*)[fullPostDocument firstNodeForXPath:commentsQueryString error:&commentsError];
            
            if (commentsError)
            {
                [errorsOccured addObject:commentsError];
                block(errorsOccured, nil);
                
                return;
            }
            
            PostFull *newPostFull = [[PostFull alloc] init];
            
            NSError  *titleError = nil;
            NSString *titleQueryString       = @"*[@class='title']/*[@class='post_title']";
            GDataXMLElement  *title          = (GDataXMLElement*)[postNode firstNodeForXPath:titleQueryString error:&titleError];
            
            if (titleError)
            {
                [errorsOccured addObject:titleError];
            }
            
            NSError  *isTranslateError = nil;
            NSString *isTranslateQueryString       = @"h1[@class='title']/*[contains(@class, 'flag flag_translation')]";
            GDataXMLElement  *isTranslation        = (GDataXMLElement*)[postNode firstNodeForXPath:isTranslateQueryString error:&isTranslateError];
            
            if (isTranslateError)
            {
                [errorsOccured addObject:isTranslateError];
            }
            
            NSError  *isRecoveryModeError = nil;
            NSString *isRecoveryModeQueryString    = @"h1[@class='title']/*[contains(@class, 'flag flag_recovery')]";
            GDataXMLElement  *isRecoveryMode       = (GDataXMLElement*)[postNode firstNodeForXPath:isRecoveryModeQueryString error:&isRecoveryModeError];
            
            if (isRecoveryModeError)
            {
                [errorsOccured addObject:isRecoveryModeError];
            }
            
            NSError  *isTutorialError = nil;
            NSString *isTutorialQueryString       = @"h1[@class='title']/*[contains(@class, 'flag flag_tutorial')]";
            GDataXMLElement  *isTutorial          = (GDataXMLElement*)[postNode firstNodeForXPath:isTutorialQueryString error:&isTutorialError];
            
            if (isTutorialError)
            {
                [errorsOccured addObject:isTutorialError];
            }
            
            NSError  *previewError = nil;
            NSString *previewQueryString         = @"div[contains(@class,'content html_format')]";
            GDataXMLElement  *preview            = (GDataXMLElement*)[postNode firstNodeForXPath:previewQueryString error:&previewError];
            
            if (previewError)
            {
                [errorsOccured addObject:previewError];
            }
            
            NSError  *timeError = nil;
            NSString *timeQueryString         = @"div[@class='published']";
            GDataXMLElement  *time            = (GDataXMLElement*)[postNode firstNodeForXPath:timeQueryString error:&timeError];
            
            if (timeError)
            {
                [errorsOccured addObject:timeError];
            }
            
            NSError  *hubsError = nil;
            NSString *hubsQueryString         = @"div[@class='hubs']";
            GDataXMLElement  *hubs            = (GDataXMLElement*)[postNode firstNodeForXPath:hubsQueryString error:&hubsError];
            
            if (hubsError)
            {
                [errorsOccured addObject:hubsError];
            }
            
            NSError  *viewsError = nil;
            NSString *viewsQueryString        = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='pageviews']";
            GDataXMLElement  *views           = (GDataXMLElement*)[postNode firstNodeForXPath:viewsQueryString error:&viewsError];
            
            if (viewsError)
            {
                [errorsOccured addObject:viewsError];
            }
            
            NSError  *favAddsError = nil;
            NSString *favAddsQueryString      = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='favs_count']";
            GDataXMLElement  *favAdds         = (GDataXMLElement*)[postNode firstNodeForXPath:favAddsQueryString error:&favAddsError];
            
            if (favAddsError)
            {
                [errorsOccured addObject:favAddsError];
            }
            
            NSError  *authorError = nil;
            NSString *authorQueryString       = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='author']/a";
            GDataXMLElement  *author          = (GDataXMLElement*)[postNode firstNodeForXPath:authorQueryString error:&authorError];
            
            if (authorError)
            {
                [errorsOccured addObject:authorError];
            }
            
            NSError  *commentsCountError = nil;
            NSString *commentsCountQueryString      = @"*[contains(@class,'title')]/*[@id='comments_count']";
            GDataXMLElement  *commentsCount         = (GDataXMLElement*)[commentsNode firstNodeForXPath:commentsCountQueryString error:&commentsCountError];
            
            if (commentsCountError)
            {
                [errorsOccured addObject:commentsCountError];
            }
            
            NSError  *ratingError = nil;
            NSString *ratingQueryString      = @"a[@class='score']|//span[@class='score']";
            GDataXMLElement  *rating         = (GDataXMLElement*)[postNode firstNodeForXPath:ratingQueryString error:&ratingError];
            
            if (ratingError)
            {
                [errorsOccured addObject:ratingError];
            }
            
            newPostFull.favAdds = [favAdds stringValue];
            
            if (newPostFull.favAdds.length == 0)
            {
                newPostFull.favAdds = @"0";
            }
            
            NSString *hubsString = @"";
            NSArray *hubsElements = [hubs children];
            
            for (GDataXMLElement *hubElement in hubsElements)
            {
                NSString *stringToAdd = [[hubElement childAtIndex:0] stringValue];
                
                if ((stringToAdd.length>0) & !([stringToAdd isEqual:@"*"]))
                {
                    hubsString = [hubsString stringByAppendingFormat:@"%@; ",stringToAdd];
                }
            }
            
            newPostFull.tags    = hubsString;
            newPostFull.title   = [title stringValue];
            
            newPostFull.url     =  revealPath;
            
            newPostFull.preview = [preview XMLString];
            newPostFull.postBodyText = [preview XMLString];
            newPostFull.date    = [time stringValue];
            newPostFull.rating  = [rating stringValue];
            newPostFull.views   = [views stringValue];
            newPostFull.author  = [author stringValue];
            newPostFull.comments = [[commentsCount stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
            
            if ([newPostFull.comments isEqualToString:@"Комментировать"])
            {
                newPostFull.comments = @"0";
            }
            
            if (isRecoveryMode)
            {
                newPostFull.isRecoveryMode = YES;
            }
            
            if (isTutorial)
            {
                newPostFull.isTutorial = YES;
            }
            
            if (isTranslation)
            {
                newPostFull.isTranslate = YES;
            }
            
            newPostFull.postCommentaries = [self commentsToPostFromNode:commentsNode];
            
            block (errorsOccured, newPostFull);
        }
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        
        if (block)
        {
            block(@[error], nil);
        }
        
    }];
    
    if (immediately)
    {
        [self enqueueHTTPRequestOperation:postFullGetOperation];
    }
    
    return postFullGetOperation;
}

- (AFHTTPRequestOperation*) getPostFullWithBlock:(void (^)(NSArray *errors, PostFull *fullPost))block
                                  fromCuttedPost:(PostCutted*)postHeader
                                startImmediately:(BOOL)immediately
{
    return [self getPostFullWithBlock:block fromRevealURLString:postHeader.url startImmediately:immediately];
}

#pragma mark - Fetching 'Questions and Answers'
- (AFHTTPRequestOperation*) getQuestionsCuttedListWithBlock:(void (^)(NSArray *errors, NSArray *questionsCutted))block
                                                 fromSource:(OAQuestionsSourceType)qSource
                                                    andPage:(int)page
                                           startImmediately:(BOOL)immediately;
{
    NSMutableArray *errorsOccured = [@[] mutableCopy];
    
    if (page < 1)
    {
        if (block)
        {
            NSError *errorWrongPage = [NSError errorWithDomain:kDomain code:-1 userInfo:@{NSLocalizedDescriptionKey:NSLocalizedString(@"Passed 'page' value is less than 1", "")}];
            [errorsOccured addObject:errorWrongPage];
            
            block(errorsOccured, nil);
        }
        
        return nil;
    }
    
    NSString *urlString;
    NSArray *questionsSources = @[kQASourceIncomingHabred, kQASourceIncomingNew, kQASourceIncomingUnHabred, kQASourceHot, kQASourceUnAnswered];
    
    urlString = [questionsSources objectAtIndex:qSource];
    urlString = [urlString stringByAppendingFormat:kQASourcePageFormatPostfix, page];
    
    NSURLRequest *questionsCuttedGetRequest = [self requestWithMethod:@"GET" path:urlString parameters:nil];
    AFHTTPRequestOperation *questionsCuttedGetOperation = [self HTTPRequestOperationWithRequest:questionsCuttedGetRequest success:^(AFHTTPRequestOperation *operation, id responseObject) {
        
        if (block)
        {
            NSError *errorWhenParse = nil;
            
            NSData   *questionsSourceHTMLData = responseObject;
            GDataXMLDocument *cuttedQuestionsDocument  = [[GDataXMLDocument alloc] initWithHTMLData:questionsSourceHTMLData error:&errorWhenParse];
            
            if (errorWhenParse)
            {
                [errorsOccured addObject:errorWhenParse];
                block(errorsOccured, nil);
                
                return;
            }
            
            //Questions wraps
            NSError  *postsError = nil;
            NSString *postsQueryString       = @"//div[@class='posts_list']/div[@class='posts shortcuts_items']/div[contains(@class,'post')]";
            NSArray  *postsNodes             = [cuttedQuestionsDocument nodesForXPath:postsQueryString error:&postsError];
            
            if (postsError)
            {
                [errorsOccured addObject:postsError];
                block(errorsOccured, nil);
                
                return;
            }
            
            NSMutableArray *newQuestionsCutted = [NSMutableArray array];
            
            for (GDataXMLElement *postNode in postsNodes)
            {
                QuestionCutted *newQuestionHeader = [[QuestionCutted alloc] init];
                
                //Titles of questions
                NSError  *titleError = nil;
                NSString *titleQueryString       = @"h1[@class='title']/a[@class='post_title']";
                GDataXMLElement  *title          = (GDataXMLElement*)[postNode firstNodeForXPath:titleQueryString error:&titleError];
                
                if (titleError)
                {
                    [errorsOccured addObject:titleError];
                }
                
                //Times when questions created
                NSError  *timeError = nil;
                NSString *timeQueryString         = @"div[@class='published']";
                GDataXMLElement  *time            = (GDataXMLElement*)[postNode firstNodeForXPath:timeQueryString error:&timeError];
                
                if (timeError)
                {
                    [errorsOccured addObject:timeError];
                }
                
                //Hubs, whole questions moved
                NSError  *hubsError = nil;
                NSString *hubsQueryString         = @"div[@class='hubs']";
                GDataXMLElement  *hubs            = (GDataXMLElement*)[postNode firstNodeForXPath:hubsQueryString error:&hubsError];
                
                if (hubsError)
                {
                    [errorsOccured addObject:hubsError];
                }
                
                //Count of questions views
                NSError  *viewsError = nil;
                NSString *viewsQueryString        = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='pageviews']";
                GDataXMLElement  *views           = (GDataXMLElement*)[postNode firstNodeForXPath:viewsQueryString error:&viewsError];
                
                if (viewsError)
                {
                    [errorsOccured addObject:viewsError];
                }
                
                //Count of adds to favorites
                NSError  *favAddsError = nil;
                NSString *favAddsQueryString      = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='favs_count']";
                GDataXMLElement  *favAdds         = (GDataXMLElement*)[postNode firstNodeForXPath:favAddsQueryString error:&favAddsError];
                
                if (favAddsError)
                {
                    [errorsOccured addObject:favAddsError];
                }
                
                //Question author
                NSError  *authorError = nil;
                NSString *authorQueryString       = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[@class='author']/a";
                GDataXMLElement  *author          = (GDataXMLElement*)[postNode firstNodeForXPath:authorQueryString error:&authorError];
                
                if (authorError)
                {
                    [errorsOccured addObject:authorError];
                }
                
                //Count of answers
                NSError  *answersError = nil;
                NSString *answersQueryString      = @"div[@class='infopanel_wrapper']/div[contains(@class,'infopanel')]/div[contains(@class,'informative')]";
                GDataXMLElement  *answers         = (GDataXMLElement*)[postNode firstNodeForXPath:answersQueryString error:&answersError];
                
                if (answersError)
                {
                    [errorsOccured addObject:answersError];
                }
                
                //Count of answers
                NSError  *ratingError = nil;
                NSString *ratingQueryString      = @"a[@class='score']|//span[@class='score']";
                GDataXMLElement  *rating         = (GDataXMLElement*)[postNode firstNodeForXPath:ratingQueryString error:&ratingError];
                
                if (ratingError)
                {
                    [errorsOccured addObject:ratingError];
                }
                
                newQuestionHeader.favAdds = [favAdds stringValue];
                
                if (newQuestionHeader.favAdds.length == 0)
                {
                    newQuestionHeader.favAdds = @"0";
                }
                
                NSString *hubsString = @"";
                NSArray *hubsElements = [hubs children];
                
                for (GDataXMLElement *hubElement in hubsElements)
                {
                    NSString *stringToAdd = [[hubElement childAtIndex:0] stringValue];
                    
                    if ((stringToAdd.length>0) & !([stringToAdd isEqual:@"*"]))
                    {
                        hubsString = [hubsString stringByAppendingFormat:@"%@; ",stringToAdd];
                    }
                }
                
                newQuestionHeader.tags    = hubsString;
                newQuestionHeader.title   = [title stringValue];
                newQuestionHeader.url     = [[title attributeForName:@"href"] stringValue];
                newQuestionHeader.date    = [time stringValue];
                newQuestionHeader.rating  = [rating stringValue];
                newQuestionHeader.views   = [views stringValue];
                newQuestionHeader.author  = [author stringValue];
                newQuestionHeader.answers = [[answers stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
                
                [newQuestionsCutted addObject:newQuestionHeader];
            }
            
            block (errorsOccured, newQuestionsCutted);
        }
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        
        if (block)
        {
            block(@[error], nil);
        }
        
    }];
    
    if (immediately)
    {
         [self enqueueHTTPRequestOperation:questionsCuttedGetOperation];
    }
    
    return questionsCuttedGetOperation;
}

- (AFHTTPRequestOperation*) getQuestionFullWithBlock:(void (^)(NSArray *errors, QuestionFull *fullQuestion))block
                                 fromRevealURLString:(NSString*)revealPath
                                    startImmediately:(BOOL)immediately;
{
    NSMutableArray *errorsOccured = [@[] mutableCopy];
    
    if (revealPath.length == 0)
    {
        if (block)
        {
            NSError *errorPathIsNil = [NSError errorWithDomain:kDomain code:-1 userInfo:@{NSLocalizedDescriptionKey:NSLocalizedString(@"Reveal path to question must not be nil", "")}];
            [errorsOccured addObject:errorPathIsNil];
            
            block(errorsOccured, nil);
        }
        
        return nil;
    }
    
    NSURLRequest *questionFullGetRequest = [self requestWithMethod:@"GET" path:revealPath parameters:nil];
    AFHTTPRequestOperation *questionFullGetOperation = [self HTTPRequestOperationWithRequest:questionFullGetRequest success:^(AFHTTPRequestOperation *operation, id responseObject) {
        
        if (block)
        {
            NSError *errorWhenParse = nil;
            
            NSData   *questionFullSourceHTMLData = responseObject;
            GDataXMLDocument *fullQuestionDocument  = [[GDataXMLDocument alloc] initWithHTMLData:questionFullSourceHTMLData error:&errorWhenParse];
            
            if (errorWhenParse)
            {
                [errorsOccured addObject:errorWhenParse];
                block(errorsOccured, nil);
                
                return;
            }
            
            //Titles of questions
            NSError  *titleError = nil;
            NSString *titleQueryString       = @"//a[@class='post_title']|//span[@class='post_title']";
            GDataXMLNode  *titleNode         = [fullQuestionDocument firstNodeForXPath:titleQueryString error:&titleError];
            
            if (titleError)
            {
                [errorsOccured addObject:titleError];
            }
            
            //Times when questions created
            NSError  *timeError = nil;
            NSString *timeQueryString         = @"//div[@class='published']";
            GDataXMLNode  *timeNode           = [fullQuestionDocument firstNodeForXPath:timeQueryString error:&timeError];
            
            if (timeError)
            {
                [errorsOccured addObject:timeError];
            }
            
            //Hubs, whole questions moved
            NSError  *hubsError = nil;
            NSString *hubsQueryString         = @"//div[@class='hubs']";
            GDataXMLNode  *hubsNode           = [fullQuestionDocument firstNodeForXPath:hubsQueryString error:&hubsError];
            
            if (hubsError)
            {
                [errorsOccured addObject:hubsError];
            }
            
            //Count of questions views
            NSError  *viewsError = nil;
            NSString *viewsQueryString        = @"//div[@class='pageviews']";
            GDataXMLNode  *viewsNode          = [fullQuestionDocument firstNodeForXPath:viewsQueryString error:&viewsError];
            
            if (viewsError)
            {
                [errorsOccured addObject:viewsError];
            }
            
            //Count of adds to favorites
            NSError  *favAddsError = nil;
            NSString *favAddsQueryString      = @"//div[@class='favs_count']";
            GDataXMLNode  *favAddsNode        = [fullQuestionDocument firstNodeForXPath:favAddsQueryString error:&favAddsError];
            
            if (favAddsError)
            {
                [errorsOccured addObject:favAddsError];
            }
            
            //Question author
            NSError  *authorError = nil;
            NSString *authorQueryString       = @"//div[@class='author']/a";
            GDataXMLNode  *authorNode         = [fullQuestionDocument firstNodeForXPath:authorQueryString error:&authorError];
            
            if (authorError)
            {
                [errorsOccured addObject:authorError];
            }
            
            //Count of answers
            NSError  *answersError = nil;
            NSString *answersQueryString      = @"//span[@id='comments_count']";
            GDataXMLNode  *answersNode        = [fullQuestionDocument firstNodeForXPath:answersQueryString error:&answersError];
            
            if (answersError)
            {
                [errorsOccured addObject:answersError];
            }
            
            //Count of answers
            NSError  *ratingError = nil;
            NSString *ratingQueryString      = @"//a[@class='score']|//span[@class='score']";
            GDataXMLNode  *ratingNode        = [fullQuestionDocument firstNodeForXPath:ratingQueryString error:&ratingError];
            
            if (ratingError)
            {
                [errorsOccured addObject:ratingError];
            }
            
            //Question HTML body
            NSError       *bodyError        = nil;
            NSString      *bodyQueryString  = @"//div[contains(@class,'content html_format')]";
            GDataXMLNode  *bodyNode         = [fullQuestionDocument firstNodeForXPath:bodyQueryString error:&bodyError];
            
            if (bodyError)
            {
                [errorsOccured addObject:bodyError];
            }
            
            //we will fetch raw xml string - all block 'content html_format with itself
            if ([bodyNode XMLString].length == 0)
            {
                NSError *errorBodyNotFound = [NSError errorWithDomain:kDomain code:-1 userInfo:@{NSLocalizedDescriptionKey:NSLocalizedString(@"Question body not found", "")}];
                
                [errorsOccured addObject:errorBodyNotFound];
            }
            
            //Question answers
            NSError   *answersArrayError        = nil;
            NSString  *answersArrayQueryString  = @"//div[@id='answers']/div[@class='answer']";
            NSArray  *answersArrayNodes         = [fullQuestionDocument nodesForXPath:answersArrayQueryString error:&answersArrayError];
            
            if (answersArrayError)
            {
                [errorsOccured addObject:answersError];
            }
            
            QuestionFull *newFullQuestion = [[QuestionFull alloc] init];
            
            NSString *hubsString = @"";
            NSArray *hubsElements = [hubsNode children];
            
            for (GDataXMLElement *hubElement in hubsElements)
            {
                NSString *stringToAdd = [[hubElement childAtIndex:0] stringValue];
                
                if ((stringToAdd.length>0) & !([stringToAdd isEqual:@"*"]))
                {
                    hubsString = [hubsString stringByAppendingFormat:@"%@; ",stringToAdd];
                }
            }
            
            newFullQuestion.tags    = hubsString;
            
            newFullQuestion.title   = [titleNode stringValue];
            newFullQuestion.url     = revealPath;
            newFullQuestion.date    = [timeNode stringValue];
            newFullQuestion.rating  = [ratingNode stringValue];
            newFullQuestion.views   = [viewsNode stringValue];
            newFullQuestion.author  = [authorNode stringValue];
            newFullQuestion.answers = [[answersNode stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
            newFullQuestion.favAdds = [favAddsNode stringValue];
            
            if (newFullQuestion.favAdds.length == 0)
            {
                newFullQuestion.favAdds = @"0";
            }
            
            newFullQuestion.questionBodyText = [bodyNode XMLString];
            
            NSMutableArray *answers = [@[] mutableCopy];
            for (GDataXMLElement *answerNode in answersArrayNodes)
            {
                NSMutableDictionary *answerDictionary = [@{} mutableCopy];
                NSMutableDictionary *authorDictionary = [@{} mutableCopy];
                NSMutableArray      *commentsArray    = [@[] mutableCopy];
                
                //WTF?! 'attributeForName:' is not declared for GDataXMLNode, but is exist! Use hack: iterate GDataXMLNodes as GDataXMLElements
                NSString *answerID = [[answerNode attributeForName:@"data-id"] stringValue];
                NSString *answerScore = [[answerNode attributeForName:@"data-score"] stringValue];
                
                if (answerID)
                {
                    [answerDictionary setObject:answerID forKey:@"id"];
                }
                
                if (answerScore)
                {
                    [answerDictionary setObject:answerScore forKey:@"score"];
                }
                
                NSError *timeError = nil;
                GDataXMLElement *timeNode = (GDataXMLElement*)[answerNode firstNodeForXPath:@"div[contains(@class,'info')]/time" error:&timeError];
                
                if (timeError)
                {
                    [errorsOccured addObject:timeError];
                }
                
                NSString *timeString = [timeNode stringValue];
                
                if (timeString)
                {
                    [answerDictionary setObject:timeString forKey:@"time"];
                }
                
                NSError *messageBodyURLError = nil;
                GDataXMLElement *messageBodyURLNode = (GDataXMLElement*)[answerNode firstNodeForXPath:@"div[contains(@class,'message html_format')]" error:&messageBodyURLError];
                
                if (messageBodyURLError)
                {
                    [errorsOccured addObject:messageBodyURLError];
                }
                
                NSString *messageBodyHTML = [messageBodyURLNode XMLString];
                
                if (messageBodyHTML)
                {
                    [answerDictionary setObject:messageBodyHTML forKey:@"body"];
                }
                
                NSError *authorAvatarURLError = nil;
                GDataXMLElement *authorAvatarURLNode = (GDataXMLElement*)[answerNode firstNodeForXPath:@"div[contains(@class,'info')]/a[@class='avatar']/img" error:&authorAvatarURLError];
                
                if (authorAvatarURLError)
                {
                    [errorsOccured addObject:authorAvatarURLError];
                }
                
                NSString *authorAvatarURLString = [[authorAvatarURLNode attributeForName:@"src"] stringValue];
                
                if (authorAvatarURLString)
                {
                    [authorDictionary setObject:authorAvatarURLString forKey:@"avatar"];
                }
                
                NSError *authorNickNameURLError = nil;
                GDataXMLElement *authorNickNameURLNode = (GDataXMLElement*)[answerNode firstNodeForXPath:@"div[contains(@class,'info')]/a[@class='username']" error:&authorNickNameURLError];
                
                if (authorNickNameURLError)
                {
                    [errorsOccured addObject:authorNickNameURLError];
                }
                
                NSString *authorNickNameURLString = [authorNickNameURLNode stringValue];
                
                if (authorNickNameURLString)
                {
                    [authorDictionary setObject:authorNickNameURLString forKey:@"username"];
                }
                
                NSError *commentsError = nil;
                NSArray *commentsNodes = [answerNode nodesForXPath:@"div[contains(@class,'comments')]/div[@class='comment_item']" error:&commentsError];
                
                if (commentsError)
                {
                    [errorsOccured addObject:commentsError];
                }
                
                for (GDataXMLElement *commentNode in commentsNodes)
                {
                    NSMutableDictionary *commentDictionary = [@{} mutableCopy];
                    
                    NSError *commentBodyError = nil;
                    GDataXMLElement *commentBodyNode = (GDataXMLElement*)[commentNode firstNodeForXPath:@"span[contains(@class,'text html_format')]" error:&commentBodyError];
                    
                    if (commentBodyError)
                    {
                        [errorsOccured addObject:commentBodyError];
                    }
                    
                    NSString *commentBodyString = [commentBodyNode stringValue];
                    
                    if (commentBodyString)
                    {
                        [commentDictionary setObject:commentBodyString forKey:@"body"];
                    }
                    
                    NSError *commentTimeError = nil;
                    GDataXMLElement *commentTimeNode = (GDataXMLElement*)[commentNode firstNodeForXPath:@"div[@class='comment_head']/span[contains(@class,'time')]" error:&commentTimeError];
                    
                    if (commentTimeError)
                    {
                        [errorsOccured addObject:commentTimeError];
                    }
                    
                    NSString *commentTimeString = [commentTimeNode stringValue];
                    
                    if (commentTimeString)
                    {
                        [commentDictionary setObject:commentTimeString forKey:@"time"];
                    }
                    
                    NSError *commentAuthorNickNameError = nil;
                    GDataXMLElement *commentAuthorNickNameNode = (GDataXMLElement*)[commentNode firstNodeForXPath:@"div[@class='comment_head']/span[contains(@class,'info')]/a"
                                                                                                  error:&commentAuthorNickNameError];
                    
                    if (commentAuthorNickNameError)
                    {
                        [errorsOccured addObject:commentAuthorNickNameError];
                    }
                    
                    NSString *commentAuthorNickNameString = [commentAuthorNickNameNode stringValue];
                    
                    if (commentAuthorNickNameString)
                    {
                        [commentDictionary setObject:@{@"username":commentAuthorNickNameString} forKey:@"author"];
                    }
                    
                    [commentsArray addObject:commentDictionary];
                }
                
                [answerDictionary setObject:authorDictionary forKey:@"author"];
                [answerDictionary setObject:commentsArray forKey:@"comments"];
                [answers addObject:answerDictionary];
            }
            
            newFullQuestion.questionAnswers = answers;
            
            block(errorsOccured, newFullQuestion);
        }
        
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        
        if (block)
        {
            block(@[error], nil);
        }
        
    }];
    
    if (immediately)
    {
        [self enqueueHTTPRequestOperation:questionFullGetOperation];
    }
    
    return questionFullGetOperation;
}

- (AFHTTPRequestOperation*) getQuestionFullWithBlock:(void (^)(NSArray *errors, QuestionFull *fullQuestion))block
                                 fromCuttedQuestion:(QuestionCutted*)questionHeader
                                    startImmediately:(BOOL)immediately;
{
    return [self getQuestionFullWithBlock:block fromRevealURLString:questionHeader.url startImmediately:immediately];
}

@end
